"use client";

import { startTransition, useActionState, useId, useState } from "react";
import { newPostAction } from "./_action";
import { Label } from "@/lib/components/ui/label";
import { Input } from "@/lib/components/ui/input";
import { Button } from "@/lib/components/ui/button";
import { Alert, AlertDescription, AlertTitle } from "@/lib/components/ui/alert";
import { Spinner } from "@/lib/components/ui/spinner";
import { InputLengthIndicator } from "@/lib/components/input-length-indicator";

export function NewPostButton() {
  const [state, action, isPending] = useActionState(newPostAction, null);
  const [error, setError] = useState<string | null>(null);

  const handleClick = async () => {
    try {
      const result = await action({});
      if (result && 'error' in result) {
        setError(result.error);
      }
    } catch (e) {
      console.error('Error creating post:', e);
      setError('予期しないエラーが発生しました');
    }
  };

  return (
    <div>
      <button onClick={handleClick} disabled={isPending}>
        {isPending ? '投稿中...' : '新規投稿'}
      </button>
      {error && <p style={{ color: 'red' }}>{error}</p>}
    </div>
  );
}
